@@ -65,4 +65,6 @@ urlpatterns = patterns(
     url(r'^image_status/$', 'image_status_list'),
     url(r'^image_status/(?P<rootfs_type>[a-zA-Z0-9_-]+)\+(?P<hwpack_type>[a-zA-Z0-9_-]+)/$', 'image_status_detail'),
     url(r'^image_status/(?P<rootfs_type>[a-zA-Z0-9_-]+)\+(?P<hwpack_type>[a-zA-Z0-9_-]+)/test-history/(?P<test_id>[^/]+)/$', 'image_test_history'),
+    url(r'^efforts/$', 'testing_effort_list'),
+    url(r'^efforts/(?P<pk>[0-9]+)/$', 'testing_effort_detail'),
 )
